package gui.entities.trainEntities;

import java.io.IOException;
import java.io.StreamTokenizer;
import globalVariables.GlobalVariables;
import gui.entities.sectionEntities.time.ReferenceTableEntry;
import gui.entities.sectionEntities.trackEntities.Loop;
import gui.entities.sectionEntityList.LoopList;

public class ScheduledTrain extends Train {

	private static final int defaultVelocity = 100;
	private static final double defaultAcceleration = 0.5;
	private static final double defaultDeceleration = 1;

	private LoopList pathLoopList = new LoopList();

	public ScheduledTrain() {
		scheduled = true;
		velocity = defaultVelocity;
		acceleration = defaultAcceleration;
		deceleration = defaultDeceleration;
		operatingDays = allOperatingDays;
	}

	public ScheduledTrain(Train simulatorTrain) {
		super(simulatorTrain);
		setOperatingDays(((ScheduledTrain) simulatorTrain).getOperatingDays());
	}

	public ScheduledTrain(int a, double b, double c, double d, double e, double f) {
		super(a, b, c, d, e, f);
	}

	public ScheduledTrain(double priority, double startTime, double length, double acceleration, double deceleration) {
		super(priority, startTime, length, acceleration, deceleration);
	}

	@Override
	public void setTrainProperties(StreamTokenizer scheduledTrainStreamTokenizer, int nTimeTableEntries)
			throws IOException {
		scheduledTrainStreamTokenizer.nextToken();
		int trainNumber = (int) scheduledTrainStreamTokenizer.nval;
		System.out.println("trainNumber " + trainNumber);

		// for special case only... remove asap TODO
		scheduledTrainStreamTokenizer.nextToken();

		// for special case only... remove asap TODO
		scheduledTrainStreamTokenizer.nextToken();

		scheduledTrainStreamTokenizer.nextToken();
		String directionString = scheduledTrainStreamTokenizer.sval;
		int direction = GlobalVariables.getDirectionFromDirectionString(directionString);
		System.out.println("directionString " + directionString);

		scheduledTrainStreamTokenizer.nextToken();
		double length = scheduledTrainStreamTokenizer.nval;
		System.out.println("length " + length);

		scheduledTrainStreamTokenizer.nextToken();
		double acceleration = scheduledTrainStreamTokenizer.nval;
		// System.out.println("acceleration " + acceleration);

		scheduledTrainStreamTokenizer.nextToken();
		double deceleration = scheduledTrainStreamTokenizer.nval;
		// System.out.println("deceleration " + deceleration);

		scheduledTrainStreamTokenizer.nextToken();
		int priority = (int) scheduledTrainStreamTokenizer.nval;
		// System.out.println("priority " + priority);

		scheduledTrainStreamTokenizer.nextToken();
		int maximumVelocity = (int) scheduledTrainStreamTokenizer.nval;
		System.out.println("abhi maximumVelocity " + maximumVelocity);

		scheduledTrainStreamTokenizer.nextToken();
		String operatingDays = scheduledTrainStreamTokenizer.sval;
		// System.out.println("operating days " + operatingDays);

		scheduledTrainStreamTokenizer.nextToken();
		nTimeTableEntries = (int) scheduledTrainStreamTokenizer.nval;

		for (int i = 0; i < nTimeTableEntries; i++) {
			scheduledTrainStreamTokenizer.nextToken();
			scheduledTrainStreamTokenizer.nextToken();
			scheduledTrainStreamTokenizer.nextToken();
		}

		// for special case only... remove asap TODO
		scheduledTrainStreamTokenizer.nextToken();

		setTrainNo(trainNumber);
		setDirection(direction);
		setAcceleration(acceleration);
		setDeceleration(deceleration);
		setPriority(priority);
		setLength(length);
		setOperatingDays(operatingDays);
		setVelocity(maximumVelocity);
	}

	public String getOperatingDays() {
		return operatingDays;
	}

	public void setOperatingDays(String operatingDays) {
		this.operatingDays = operatingDays;
	}

	public LoopList getPathLoopList() {
		return pathLoopList;
	}

	public void setPathLoopList(LoopList pathLoopList) {
		this.pathLoopList = pathLoopList;
	}

	public void readReferenceTableEntries(StreamTokenizer streamTokenizer, int nEntries) throws IOException {
		for (int i = 0; i < nEntries; i++) {

			streamTokenizer.nextToken();
			ReferenceTableEntry referenceTableEntry = new ReferenceTableEntry();
			referenceTableEntry.refLoopNo = (int) streamTokenizer.nval;
			String[] LoopType = Loop.loopNumToDirType.get((int) streamTokenizer.nval).split("-");
			if (LoopType[0].equals("common"))
				referenceTableEntry.loopDir = "Com";
			else
				referenceTableEntry.loopDir = LoopType[0].substring(0, 1).toUpperCase() + LoopType[0].substring(1); // Abhishek
			if (LoopType[1].equals("loop"))
				referenceTableEntry.loopType = "LL"; // Abhishek
			else
				referenceTableEntry.loopType = "ML"; // Abhishek
			referenceTableEntry.setStationName(Loop.loopNumToStn.get((int) streamTokenizer.nval)); // Abhishek

			System.out.println("loop num_dir_type" + referenceTableEntry.refLoopNo + "_" + referenceTableEntry.loopDir + "_"
					+ referenceTableEntry.loopType);

			streamTokenizer.nextToken();
			referenceTableEntry.refArrTimeInput = (int) streamTokenizer.nval;
			// System.out.println(referenceTableEntry.refArrTimeInput);

			streamTokenizer.nextToken();
			referenceTableEntry.refDepTimeInput = (int) streamTokenizer.nval;

			refTables.add(referenceTableEntry);
		}

	}

	// sneha added following function
	public void readTempReferenceTableEntries(StreamTokenizer streamTokenizer, int nEntries) throws IOException {

		for (int i = 0; i < nEntries; i++) {

			streamTokenizer.nextToken();
			ReferenceTableEntry referenceTableEntry = new ReferenceTableEntry();
			referenceTableEntry.refLoopNo = (int) streamTokenizer.nval;

			streamTokenizer.nextToken();
			referenceTableEntry.setReferenceArrivalTime(streamTokenizer.nval);

			streamTokenizer.nextToken();
			referenceTableEntry.setReferenceDepartureTime(streamTokenizer.nval);
			// System.out.println(referenceTableEntry.refLoopNo + "\t"
			// +referenceTableEntry.getReferenceArrivalTime()
			// +"\t"+referenceTableEntry.getReferenceDepartureTime());

			tempRefTables.add(referenceTableEntry);
		}

	}

}
