package input;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Scanner;

import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import files.Tweak;
import globalVariables.FileNames;
import globalVariables.GlobalVar;

/**
 * 
 * @author Abhishek
 *
 */
@SuppressWarnings("serial")
public class TweakInputDialog extends InputDialog {

	public static JFrame tweakFrame = null;
	public static JPanel panel = new JPanel();
	public static ArrayList<String> locotypeListArray = new ArrayList<String>();
	public static String[] locotypeList = { "Select", "WDM 2-3000", "WDG 4-3500", "WAP 7-4000", "WAG 7-4500", "WAG 9-5000", "Add More" };
	public static JComboBox<String> locolist = new JComboBox<String>(locotypeList);
	public static JLabel TELabel = new JLabel("TE (equi. in Tonnes)");
	public static JTextField TEField = new JTextField("");
	public static JLabel decLabel = new JLabel("Deceleration");
	public static JTextField decField = new JTextField("");
	public static JLabel speedLabel = new JLabel("Maximum Speed");
	public static JTextField speedField = new JTextField("");
	public static JLabel loadLabel = new JLabel("Load (Tonnes)");
	public static JTextField loadField = new JTextField("");
	public static JLabel accLabel = new JLabel("Acceleration");
	public static JTextField accField = new JTextField("");
	public static JLabel locoTypeLabel = new JLabel("Loco-Type");
	public static JTextField locoTypeField = new JTextField("");
	public static JButton update = new JButton("Update");
	public static JButton save = new JButton("Save");
	public static JButton add = new JButton("Add");
	public int EdenHazard;
	public static JLabel resultLabel = new JLabel("");

	public static int flag = 0;

	public TweakInputDialog() {
		tweakFrame = new JFrame();
		tweakFrame.setUndecorated(true);
		tweakFrame.getRootPane().setWindowDecorationStyle(JRootPane.INFORMATION_DIALOG);
		tweakFrame.setTitle("Tweak loco-type parameters");
		tweakFrame.setBounds(x + 10, y + 10, 480, 350);

		try {
			tweakFrame.setIconImage(ImageIO.read(new File(System.getProperty("user.dir") + "/res/logo.png")));
		} catch (IOException exc) {
			// exc.printStackTrace();
		}

		tweakFrame.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		accField.setEditable(false);

		addComponents();
		setComponentBounds();
		setComponentActionListeners();
		setComponentTextToolTip();

		panel.setLayout(null);
		tweakFrame.add(panel);
		locolist.setSelectedItem(TrainInputDialog.locolist.getSelectedItem());
		System.out.println(TrainInputDialog.locolist.getSelectedItem());
		add.setVisible(false);
		locoTypeLabel.setVisible(false);
		locoTypeField.setVisible(false);
		
		addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent windowEvent) {
				GlobalVar.tweakInputDialog = null;
		    }
		});
	}

	/**
	 * read loco-details.txt file
	 */
	public static void readLocoFile() {
		File f = new File(FileNames.getlocoFilename());
		if (!FileNames.getTestCaseDirectory().isEmpty() && !f.exists()) {
			setFields();
			writeLocoFile();
		}
		TrainInputDialog.tweakList.clear();
		locotypeListArray.clear();
		try {
			StreamTokenizer streamTokenizer = new StreamTokenizer(new FileReader(FileNames.getlocoFilename()));
			streamTokenizer.slashSlashComments(true);
			streamTokenizer.slashStarComments(true);
			while (streamTokenizer.nextToken() != StreamTokenizer.TT_EOF) {
				String[] locoFull = streamTokenizer.sval.split("-");
				locotypeListArray.add(streamTokenizer.sval);

				String locoType = locoFull[0];

				streamTokenizer.nextToken();
				double TractiveEffort = streamTokenizer.nval;

				streamTokenizer.nextToken();
				double deceleration = streamTokenizer.nval;

				streamTokenizer.nextToken();
				double maximumSpeed = streamTokenizer.nval;

				double load = Double.valueOf(locoFull[1]);

				Tweak tweak = new Tweak(locoType, TractiveEffort, deceleration, maximumSpeed, load);
				TrainInputDialog.tweakList.add(tweak);
			}
			locotypeList = new String[locotypeListArray.size() + 2];
			locotypeList[0] = "Select";
			for (int i = 0; i < locotypeListArray.size(); i++) {
				locotypeList[i + 1] = locotypeListArray.get(i);
			}
			locotypeList[locotypeListArray.size() + 1] = "Add More";
			//locolist = new JComboBox<String>(locotypeList);
			locolist.setModel(new DefaultComboBoxModel<String>(locotypeList));
		} catch (IOException e) {
			System.out.println("No Directory");
		}
	}

	/**
	 * write to loco-details.txt file
	 */
	public static void writeLocoFile() {
		BufferedWriter bw;
		try {
			bw = new BufferedWriter(new FileWriter(FileNames.getlocoFilename()));
			bw.write("/*LocoType-Load Tractive-Effort(Equi.InTonnes) Deceleration(m/s^2) MaxSpeed(kmph)*/\n");

			for (Tweak tweak : TrainInputDialog.tweakList) {
				bw.write("\"" + tweak.locoType + "-" + String.valueOf((int) tweak.load) + "\"");
				bw.write(" ");
				bw.write(String.valueOf(tweak.TractiveEffort));
				bw.write(" ");
				bw.write(String.valueOf(tweak.deceleration));
				bw.write(" ");
				bw.write(String.valueOf(tweak.maximumSpeed));
				bw.write("\n");
			}
			bw.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void addComponents() {
		panel.add(locolist);
		panel.add(TELabel);
		panel.add(TEField);
		panel.add(decLabel);
		panel.add(decField);
		panel.add(speedLabel);
		panel.add(speedField);
		panel.add(loadLabel);
		panel.add(loadField);
		panel.add(accLabel);
		panel.add(locoTypeLabel);
		panel.add(locoTypeField);
		panel.add(accField);
		panel.add(update);
		panel.add(save);
		panel.add(add);
		panel.add(resultLabel);
	}

	@Override
	public void setComponentBounds() {
		int x = 20;
		int y = 20;
		int labelWidth = 150;
		int labelHeight = 30;
		int buttonWidth = 100;
		int buttonHeight = 20;
		int fieldWidth = 100;
		int fieldHeight = 20;
		int x1 = x + labelWidth + 20;
		int ydifference = 25;

		locolist.setBounds(x, y, fieldWidth, fieldHeight);

		y += ydifference + 20;
		TELabel.setBounds(x, y, labelWidth, labelHeight);
		TEField.setBounds(x1, y, fieldWidth, fieldHeight);

		y += ydifference;
		decLabel.setBounds(x, y, labelWidth, labelHeight);
		decField.setBounds(x1, y, fieldWidth, fieldHeight);

		y += ydifference;
		speedLabel.setBounds(x, y, labelWidth, labelHeight);
		speedField.setBounds(x1, y, fieldWidth, fieldHeight);

		y += ydifference;
		loadLabel.setBounds(x, y, labelWidth, labelHeight);
		loadField.setBounds(x1, y, fieldWidth, fieldHeight);
		
		y += ydifference;
		accLabel.setBounds(x, y, labelWidth, labelHeight);
		accField.setBounds(x1, y, fieldWidth, fieldHeight);
		locoTypeLabel.setBounds(x, y, labelWidth, labelHeight);
		locoTypeField.setBounds(x1, y, fieldWidth, fieldHeight);

		y += ydifference + 30;
		update.setBounds(x, y, buttonWidth, buttonHeight);
		save.setBounds(x1, y, buttonWidth, buttonHeight);
		add.setBounds(x, y, buttonWidth, buttonHeight);

		y += ydifference;
		resultLabel.setBounds(x, y, labelWidth + 100, labelHeight);
	}

	/**
	 * Assigning funtion to dropdown menu, update button and save button 
	 */
	@Override
	public void setComponentActionListeners() {
		locolist.addItemListener(StateChangeListener);
		TEField.getDocument().addDocumentListener(new DocumentListener() {
			  public void changedUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void removeUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void insertUpdate(DocumentEvent e) {
			    changed();
			  }
			});
		decField.getDocument().addDocumentListener(new DocumentListener() {
			  public void changedUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void removeUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void insertUpdate(DocumentEvent e) {
			    changed();
			  }
			});
		speedField.getDocument().addDocumentListener(new DocumentListener() {
			  public void changedUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void removeUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void insertUpdate(DocumentEvent e) {
			    changed();
			  }
			});
		loadField.getDocument().addDocumentListener(new DocumentListener() {
			  public void changedUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void removeUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void insertUpdate(DocumentEvent e) {
			    changed();
			  }
			});
		locoTypeField.getDocument().addDocumentListener(new DocumentListener() {
			  public void changedUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void removeUpdate(DocumentEvent e) {
			    changed();
			  }
			  public void insertUpdate(DocumentEvent e) {
			    changed();
			  }
			});
		update.addActionListener(updateActionListener);
		save.addActionListener(saveActionListener);
		add.addActionListener(addLocoLoadsActionListener);
	}
	
	 public void changed() {
		 String X = String.valueOf(locolist.getSelectedItem());
		 if (X.equals("Add More")) {
			add.setEnabled(false);
			resultLabel.setText("");
			if (TEField.getText().isEmpty() || decField.getText().isEmpty() || speedField.getText().isEmpty() || loadField.getText().isEmpty() || locoTypeField.getText().isEmpty()) {
				resultLabel.setText("Please fill all the fields");
				return;
			}
			try {
				Double.valueOf(TEField.getText());
				Double.valueOf(decField.getText());
				Double.valueOf(speedField.getText());
				Double.valueOf(loadField.getText());
			} catch (Exception exp) {
				resultLabel.setText("Fields must be numerals");
				return;
			}
			if (Double.valueOf(TEField.getText()) <= 0 || Double.valueOf(decField.getText()) <= 0 || Double.valueOf(speedField.getText()) <= 0 || Double.valueOf(loadField.getText()) <= 0) {
				resultLabel.setText("Values greater than 0 are acceptable");
				return;
			}
			for (Tweak tweak : TrainInputDialog.tweakList) {
				if (tweak.locoType.equals(locoTypeField.getText()) && tweak.load == Double.valueOf(loadField.getText())) {
					resultLabel.setText("Loco-Load combination already exists");
					add.setEnabled(false);
					return;
				}
			}
			flag = 0;
			File f = new File(FileNames.getlocoFilename());
			if (f.exists() && add.isEnabled())
				save.setEnabled(true);
		}
		add.setEnabled(true);
		accField.setText("");
		DecimalFormat df = new DecimalFormat("#.###");
		df.setRoundingMode(RoundingMode.CEILING);
		if (TEField.getText().isEmpty() || decField.getText().isEmpty() || speedField.getText().isEmpty() || loadField.getText().isEmpty()) {
			resultLabel.setText("Fields cannot be empty");
			return;
		}
		try {
			Double.valueOf(TEField.getText());
			Double.valueOf(decField.getText());
			Double.valueOf(speedField.getText());
			Double.valueOf(loadField.getText());
		} catch (Exception exp) {
			resultLabel.setText("Fields must be numerals");
			return;
		}
		if (Double.valueOf(TEField.getText()) <= 0 || Double.valueOf(decField.getText()) <= 0 || Double.valueOf(speedField.getText()) <= 0 || Double.valueOf(loadField.getText()) <= 0) {
			resultLabel.setText("Values greater than 0 are acceptable");
			return;
		}
		resultLabel.setText("");
		double TE = Double.valueOf(TEField.getText());
		double load = Double.valueOf(loadField.getText());
		double accel = TE * GlobalVar.g / load; 
		accField.setText(String.valueOf(df.format(accel)));
	}
	
	/**
	 * dropdown menu function declaration
	 */
	ItemListener StateChangeListener = new ItemListener() {
		@Override
		public void itemStateChanged(ItemEvent arg0) {
			if (locolist.getSelectedItem().equals("Select")) {
				TEField.setVisible(false);
				decField.setVisible(false);
				speedField.setVisible(false);
				loadField.setVisible(false);
				accLabel.setVisible(true);
				accField.setVisible(false);
				update.setVisible(true);
				add.setVisible(false);
				locoTypeLabel.setVisible(false);
				locoTypeField.setVisible(false);
			}
			else if (locolist.getSelectedItem().equals("Add More")) {
				TEField.setVisible(true);
				TEField.setText("");
				decField.setVisible(true);
				decField.setText("");
				speedField.setVisible(true);
				speedField.setText("");
				loadField.setVisible(true);
				loadField.setText("");
				accLabel.setVisible(false);
				accField.setVisible(false);
				update.setVisible(false);
				add.setVisible(true);
				add.setEnabled(false);
				locoTypeLabel.setVisible(true);
				locoTypeField.setVisible(true);
				locoTypeField.setText("");
			}
			else {
				TEField.setVisible(true);
				decField.setVisible(true);
				speedField.setVisible(true);
				loadField.setVisible(true);
				accLabel.setVisible(true);
				accField.setVisible(true);
				update.setVisible(true);
				add.setVisible(false);
				locoTypeLabel.setVisible(false);
				locoTypeField.setVisible(false);
			}

			setFields();
		}
	};
	
	private void setComponentTextToolTip() {
		TELabel.setToolTipText("Tractive Effort Temp Text");
		TEField.setToolTipText("Tractive Effort Temp Text");
		loadLabel.setToolTipText("Load Temp Text");
		loadField.setToolTipText("Load Temp Text");
		accLabel.setToolTipText("TE x g / Load");
		accField.setToolTipText("TE x g / Load");
		update.setToolTipText("Update the above parameters for repeated use");
		save.setToolTipText("Write the above parameters in the loco-typeDetails.txt file");
	}

	/**
	 * update button function listener
	 */
	ActionListener updateActionListener = new ActionListener() {
		@SuppressWarnings("unchecked")
		@Override
		public void actionPerformed(ActionEvent e) {
			resultLabel.setText("");
			if (TEField.getText().isEmpty() || decField.getText().isEmpty() || speedField.getText().isEmpty() || loadField.getText().isEmpty()) {
				resultLabel.setText("Fields cannot be empty");
				return;
			}
			try {
				Double.valueOf(TEField.getText());
				Double.valueOf(decField.getText());
				Double.valueOf(speedField.getText());
				Double.valueOf(loadField.getText());
			} catch (Exception exp) {
				resultLabel.setText("Fields must be numerals");
				return;
			}
			if (Double.valueOf(TEField.getText()) <= 0 || Double.valueOf(decField.getText()) <= 0 || Double.valueOf(speedField.getText()) <= 0 || Double.valueOf(loadField.getText()) <= 0) {
				resultLabel.setText("Values greater than 0 are acceptable");
				return;
			}
			flag = 0;
			File f = new File(FileNames.getlocoFilename());
			if (f.exists())
				save.setEnabled(true);

			String X = String.valueOf(locolist.getSelectedItem());
			int index = 1;
			for (Tweak tweak : TrainInputDialog.tweakList) {
				if (tweak.locoType.equals(X.split("-")[0]) && tweak.load == Double.valueOf(X.split("-")[1])) {
					tweak.acceleration = Double.valueOf(TEField.getText()) * GlobalVar.g / Double.valueOf(loadField.getText());
					tweak.TractiveEffort = Double.valueOf(TEField.getText());
					tweak.deceleration = Double.valueOf(decField.getText());
					tweak.maximumSpeed = Double.valueOf(speedField.getText());
					tweak.load = Double.valueOf(loadField.getText());
					locotypeList[index] = tweak.locoType + "-" + Double.valueOf(loadField.getText()).intValue();
					TrainInputDialog.locotypeList[index] = locotypeList[index];
					locolist.setModel(new DefaultComboBoxModel<String>(locotypeList));
					TrainInputDialog.locolist.setModel(new DefaultComboBoxModel<String>(locotypeList));
					locolist.setSelectedItem(locotypeList[index]);
					break;
				}
				index++;
			}
			TrainInputDialog.locolist.setSelectedIndex(TrainInputDialog.index);
		}
	};
	
	ActionListener addLocoLoadsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			/*
			 * Loco Name Processing
			 */
			String tempName = "";
			if (locoTypeField.getText().contains(" ")) {
				String[] NameWithSpace = locoTypeField.getText().split(" ");
				String NameWithoutSpace = "";
				//System.out.println("Removing spaces");
				for (int i = 0; i < NameWithSpace.length; i++) {
					//System.out.println(NameWithSpace[i]);
					NameWithoutSpace += NameWithSpace[i];
					//System.out.println("Name Formed: " + NameWithoutSpace);
				}
				tempName = NameWithoutSpace;
				//System.out.println(tempName);
			}
			if (tempName.contains("-")) {
				String[] NameWithHyphen = tempName.split("-");
				String NameWithoutHyphen = "";
				//System.out.println("Removing hyphens");
				for (int i = 0; i < NameWithHyphen.length; i++) {
					//System.out.println(NameWithHyphen[i]);
					NameWithoutHyphen += NameWithHyphen[i];
					//System.out.println("Name Formed: " + NameWithoutHyphen);
				}
				tempName = NameWithoutHyphen;
				//System.out.println(tempName);
			}
			if (tempName == "") {
				tempName = locoTypeField.getText();
			}
			String[] locoNameFull = tempName.split("\\d+", 2);
			String locoName = tempName.toUpperCase();
			if (locoNameFull.length > 1)
				locoName = locoNameFull[0].trim().toUpperCase() + " " + String.valueOf(new Scanner(tempName).useDelimiter("\\D+").nextInt()) + locoNameFull[1].trim();
			/*
			 * Name processing ends here :)
			 */
			Tweak tweak = new Tweak(locoName, Double.valueOf(TEField.getText()), Double.valueOf(decField.getText()), Double.valueOf(speedField.getText()), Double.valueOf(loadField.getText()));
			TrainInputDialog.tweakList.add(tweak);
			locotypeList = new String[TrainInputDialog.tweakList.size() + 2];
			locotypeList[0] = "Select";
			for (int i = 0; i < TrainInputDialog.tweakList.size(); i++) {
				locotypeList[i + 1] = TrainInputDialog.tweakList.get(i).locoType + "-" + String.valueOf((int) TrainInputDialog.tweakList.get(i).load);
			}
			locotypeList[TrainInputDialog.tweakList.size() + 1] = "Add More";
			locolist.setModel(new DefaultComboBoxModel<String>(locotypeList));
			TrainInputDialog.locolist.setModel(new DefaultComboBoxModel<String>(locotypeList));
			locoTypeField.setText("");
			save.setEnabled(true);
			locolist.setSelectedIndex(TrainInputDialog.tweakList.size());
		}
	};

	/**
	 * save button function declaration
	 */
	ActionListener saveActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			writeLocoFile();
			save.setEnabled(false);
		}
	};

	/**
	 * default/read from file values to be populated into text-fields.
	 */
	protected static void setFields() {
		if (TrainInputDialog.tweakList.isEmpty()) {
			Tweak tweak = new Tweak("WDM 2", 26, 1, 110, 3000);
			TrainInputDialog.tweakList.add(tweak);
			tweak = new Tweak("WDG 4", 53, 0.9, 100, 3500);
			TrainInputDialog.tweakList.add(tweak);
			tweak = new Tweak("WAP 7", 32.8, 0.8, 140, 4000);
			TrainInputDialog.tweakList.add(tweak);
			tweak = new Tweak("WAG 7", 44, 0.95, 100, 4500);
			TrainInputDialog.tweakList.add(tweak);
			tweak = new Tweak("WAG 9", 46.9, 1.05, 100, 5000);
			TrainInputDialog.tweakList.add(tweak);
		}
		String X = String.valueOf(locolist.getSelectedItem());
		for (Tweak tweak1 : TrainInputDialog.tweakList) {
			if (tweak1.locoType.equals(X.split("-")[0])) {
				TEField.setText(String.valueOf(tweak1.TractiveEffort));
				decField.setText(String.valueOf(tweak1.deceleration));
				speedField.setText(String.valueOf(tweak1.maximumSpeed));
				loadField.setText(String.valueOf(tweak1.load));
				break;
			}
		}
	}
}