package simulator.outputFeatures;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JRootPane;

import simulator.dispatcher.FreightSimulator;
import simulator.input.SimulationInstance;
import simulator.outputFeatures.graph.Paper;

public class GraphFrame extends JFrame {
	private static final long serialVersionUID = 1L;

	/**
	 * graphPanel
	 */
	GraphPanel graphPanel;

	public SearchFrame searchFrame = null;

	/**
	 * @param freightSimulator
	 */
	public GraphFrame(FreightSimulator freightSimulator, final SimulationInstance simulationInstance) {
		super("Graph");
		javax.swing.JFrame.setDefaultLookAndFeelDecorated(true);
		javax.swing.JDialog.setDefaultLookAndFeelDecorated(true);
		this.setFocusable(true);
		GraphKeyListener graphKeyListener = new GraphKeyListener(this, simulationInstance);
		graphPanel = new GraphPanel(freightSimulator, this, graphKeyListener, simulationInstance);
		graphKeyListener.trainGraph = graphPanel.trainGraph;
		this.addKeyListener(graphKeyListener);
		getContentPane().setLayout(null);

		graphPanel.setBounds(0, 0, 1200, 1200);
		graphPanel.addKeyListener(graphKeyListener);
		setBounds(20, 20, 1150, 700);
		getContentPane().add(graphPanel);
		getContentPane().addKeyListener(new GraphKeyListener());

		try {
			this.setIconImage(ImageIO.read(new File(System.getProperty("user.dir") + "/res/logo.png")));
		} catch (IOException exc) {
			// exc.printStackTrace();
		}
		requestFocusInWindow();

		addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				if (simulationInstance.hasRailClipse == false)
					System.exit(0);
			}
		});
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				graphPanel.setBounds(0, 0, getWidth(), getHeight());
			}
		});

		setVisible(true);
	}

	/**
	 * @return {@link Paper}
	 */
	public Paper getPaper() {
		return graphPanel.getPaper();
	}
}